@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
 
-# Copyright (c) 2002-2020 "Neo4j,"
+# Copyright (c) "Neo4j"
 # Neo4j Sweden AB [http://neo4j.com]
 #
 # This file is part of Neo4j.
@@ -29,6 +29,7 @@ __all__ = [
     "AuthToken",
     "basic_auth",
     "kerberos_auth",
+    "bearer_auth",
     "custom_auth",
     "Bookmark",
     "ServerInfo",
@@ -69,6 +70,7 @@ from neo4j.api import (
     AuthToken,
     basic_auth,
     kerberos_auth,
+    bearer_auth,
     custom_auth,
     Bookmark,
     ServerInfo,
@@ -317,21 +319,28 @@ class Driver:
         Use the exception to further understand the cause of the connectivity problem.
 
         Note: Even if this method throws an exception, the driver still need to be closed via close() to free up all resources.
+
+        .. note::
+            \\*\\*config is experimental may change in the future.
+
         """
         raise NotImplementedError
 
-    @experimental("Feature support query, based on Bolt Protocol Version and Neo4j Server Version will change in the future.")
+    @experimental("Feature support query, based on Bolt protocol version and Neo4j server version will change in the future.")
     def supports_multi_db(self):
         """ Check if the server or cluster supports multi-databases.
 
         :return: Returns true if the server or cluster the driver connects to supports multi-databases, otherwise false.
         :rtype: bool
-        """
-        cx = self._pool.acquire(access_mode=READ_ACCESS, timeout=self._pool.workspace_config.connection_acquisition_timeout, database=self._pool.workspace_config.database)
-        support = cx.supports_multiple_databases
-        self._pool.release(cx)
 
-        return support
+
+        .. note::
+            Feature support query, based on Bolt Protocol Version and Neo4j
+            server version will change in the future.
+        """
+        with self.session() as session:
+            session._connect(READ_ACCESS)
+            return session._connection.supports_multiple_databases
 
 
 class BoltDriver(Direct, Driver):
@@ -435,17 +444,26 @@ class Neo4jDriver(Routing, Driver):
         return self._verify_routing_connectivity()
 
     def _verify_routing_connectivity(self):
-        from neo4j.exceptions import ServiceUnavailable
-        from neo4j._exceptions import BoltHandshakeError
+        from neo4j.exceptions import (
+            Neo4jError,
+            ServiceUnavailable,
+            SessionExpired,
+        )
 
         table = self._pool.get_routing_table_for_default_database()
         routing_info = {}
         for ix in list(table.routers):
             try:
-                routing_info[ix] = self._pool.fetch_routing_info(address=table.routers[0], timeout=self._default_workspace_config.connection_acquisition_timeout, database=self._default_workspace_config.database)
-            except BoltHandshakeError as error:
+                routing_info[ix] = self._pool.fetch_routing_info(
+                    address=table.routers[0],
+                    database=self._default_workspace_config.database,
+                    imp_user=self._default_workspace_config.impersonated_user,
+                    bookmarks=None,
+                    timeout=self._default_workspace_config
+                                .connection_acquisition_timeout
+                )
+            except (ServiceUnavailable, SessionExpired, Neo4jError):
                 routing_info[ix] = None
-
         for key, val in routing_info.items():
             if val is not None:
                 return routing_info
