@@ -30,8 +30,12 @@ class FrontendError(Exception):
 def load_config():
     with open(path.join(path.dirname(__file__), "test_config.json"), "r") as fd:
         config = json.load(fd)
-    return (config["skips"],
-            [k for k, v in config["features"].items() if v is True])
+    skips = config["skips"]
+    features = [k for k, v in config["features"].items() if v is True]
+    import ssl
+    if ssl.HAS_TLSv1_3:
+        features += ["Feature:TLS:1.3"]
+    return skips, features
 
 
 SKIPPED_TESTS, FEATURES = load_config()
@@ -54,21 +58,44 @@ def NewDriver(backend, data):
     data["authorizationToken"].mark_item_as_read_if_equals(
         "name", "AuthorizationToken"
     )
-    auth = neo4j.Auth(
-            auth_token["scheme"], auth_token["principal"],
-            auth_token["credentials"], realm=auth_token["realm"])
-    auth_token.mark_item_as_read_if_equals("ticket", "")
-    resolver = None
+    scheme = auth_token["scheme"]
+    if scheme == "basic":
+        auth = neo4j.basic_auth(
+            auth_token["principal"], auth_token["credentials"],
+            realm=auth_token.get("realm", None)
+        )
+    elif scheme == "kerberos":
+        auth = neo4j.kerberos_auth(auth_token["credentials"])
+    elif scheme == "bearer":
+        auth = neo4j.bearer_auth(auth_token["credentials"])
+    else:
+        auth = neo4j.custom_auth(
+            auth_token["principal"], auth_token["credentials"],
+            auth_token["realm"], auth_token["scheme"],
+            **auth_token.get("parameters", {})
+        )
+        auth_token.mark_item_as_read("parameters", recursive=True)
+    kwargs = {}
     if data["resolverRegistered"] or data["domainNameResolverRegistered"]:
-        resolver = resolution_func(backend, data["resolverRegistered"],
-                                   data["domainNameResolverRegistered"])
-    connection_timeout = data.get("connectionTimeoutMs", None)
-    if connection_timeout is not None:
-        connection_timeout /= 1000
+        kwargs["resolver"] = resolution_func(
+            backend, data["resolverRegistered"],
+            data["domainNameResolverRegistered"]
+        )
+    if data.get("connectionTimeoutMs"):
+        kwargs["connection_timeout"] = data["connectionTimeoutMs"] / 1000
+    if data.get("maxTxRetryTimeMs"):
+        kwargs["max_transaction_retry_time"] = data["maxTxRetryTimeMs"] / 1000
+    if data.get("connectionAcquisitionTimeoutMs"):
+        kwargs["connection_acquisition_timeout"] = \
+            data["connectionAcquisitionTimeoutMs"] / 1000
+    if data.get("maxConnectionPoolSize"):
+        kwargs["max_connection_pool_size"] = data["maxConnectionPoolSize"]
+    if data.get("fetchSize"):
+        kwargs["fetch_size"] = data["fetchSize"]
+
     data.mark_item_as_read("domainNameResolverRegistered")
     driver = neo4j.GraphDatabase.driver(
-        data["uri"], auth=auth, user_agent=data["userAgent"],
-        resolver=resolver, connection_timeout=connection_timeout
+        data["uri"], auth=auth, user_agent=data["userAgent"], **kwargs
     )
     key = backend.next_key()
     backend.drivers[key] = driver
@@ -178,12 +205,14 @@ def NewSession(backend, data):
     elif access_mode == "w":
         access_mode = neo4j.WRITE_ACCESS
     else:
-        raise Exception("Unknown access mode:" + access_mode)
+        raise ValueError("Unknown access mode:" + access_mode)
     config = {
             "default_access_mode": access_mode,
             "bookmarks": data["bookmarks"],
             "database": data["database"],
-            "fetch_size": data.get("fetchSize", None)
+            "fetch_size": data.get("fetchSize", None),
+            "impersonated_user": data.get("impersonatedUser", None),
+
     }
     session = driver.session(**config)
     key = backend.next_key()
@@ -291,6 +320,13 @@ def TransactionRollback(backend, data):
     backend.send_response("Transaction", {"id": key})
 
 
+def TransactionClose(backend, data):
+    key = data["txId"]
+    tx = backend.transactions[key]
+    tx.close()
+    backend.send_response("Transaction", {"id": key})
+
+
 def ResultNext(backend, data):
     result = backend.results[data["resultId"]]
     try:
@@ -301,6 +337,28 @@ def ResultNext(backend, data):
     backend.send_response("Record", totestkit.record(record))
 
 
+def ResultSingle(backend, data):
+    result = backend.results[data["resultId"]]
+    backend.send_response("Record", totestkit.record(result.single()))
+
+
+def ResultPeek(backend, data):
+    result = backend.results[data["resultId"]]
+    record = result.peek()
+    if record is not None:
+        backend.send_response("Record", totestkit.record(record))
+    else:
+        backend.send_response("NullRecord", {})
+
+
+def ResultList(backend, data):
+    result = backend.results[data["resultId"]]
+    records = list(result)
+    backend.send_response("RecordList", {
+        "records": [totestkit.record(r) for r in records]
+    })
+
+
 def ResultConsume(backend, data):
     result = backend.results[data["resultId"]]
     summary = result.consume()
@@ -363,8 +421,7 @@ def ForcedRoutingTableUpdate(backend, data):
     database = data["database"]
     bookmarks = data["bookmarks"]
     with driver._pool.refresh_lock:
-        driver._pool.create_routing_table(database)
-        driver._pool.update_routing_table(database=database,
+        driver._pool.update_routing_table(database=database, imp_user=None,
                                           bookmarks=bookmarks)
     backend.send_response("Driver", {"id": driver_id})
 
